@@ -25,7 +25,7 @@
   Dependencies.
 */
 
-typedef enum { FLAGT, PARMT, STRINGPARMT } Opttype;  /* type of dependent */
+typedef enum { FLAGT, PARMT, FLOATPARMT, STRINGPARMT } Opttype;
 typedef struct optdep * Optdep;
 
 struct optdep {  /* these get attached to flags/parms that have dependents */
@@ -33,6 +33,7 @@ struct optdep {  /* these get attached to flags/parms that have dependents */
   Opttype type;        /* type of dependent option */
   int id;              /* id of dependent option */
   int val;             /* value for dependent flag or parm */
+  double dval;         /* value for dependent floatparm */
   char *sval;          /* value for dependent stringparm */
   BOOL multiply;       /* means val is multiplier instead of fixed value */
   Optdep next;         /* next dependent */
@@ -53,6 +54,14 @@ struct parm {  /* Parms are integer valued options */
   Optdep dependencies;
 };
 
+struct floatparm {  /* FloatParms are double-valued options */
+  char *name;
+  double val;
+  double default_val;
+  double min, max;  /* minimum and maximum permissible values */
+  Optdep dependencies;
+};
+
 struct stringparm {  /* Stringparms are string valued options */
   char *name;
   char *val;      /* current valuse: index into array of strings */
@@ -64,10 +73,12 @@ struct stringparm {  /* Stringparms are string valued options */
 
 static struct flag Flags[MAX_FLAGS];
 static struct parm Parms[MAX_PARMS];
+static struct floatparm Floatparms[MAX_FLOATPARMS];
 static struct stringparm Stringparms[MAX_STRINGPARMS];
 
 static int Next_flag = 0;
 static int Next_parm = 0;
+static int Next_floatparm = 0;
 static int Next_stringparm = 0;
 
 static int Option_updates = 0;  /* Flag, Parm, Stringparm */
@@ -274,6 +285,41 @@ int init_parm(char *name,
   return id;
 }  /* init_parm */
 
+/*************
+ *
+ *    init_floatparm()
+ *
+ *************/
+
+/* DOCUMENTATION
+Initialize a floatparmparm (double-valued option).
+You give it a name, a default value, and  min and max values.
+An integer id is returned.
+Parms are typically
+changed by user commands which are parsed by read_commands().
+The value of a parm is checked with floatparm(ID).
+*/
+
+/* PUBLIC */
+int init_floatparm(char *name,
+		   double default_value,
+		   double min_value,
+		   double max_value)
+{
+  int id = -1;
+  if (Next_floatparm == MAX_FLOATPARMS)
+    fatal_error("init_floatparm: too many parms");
+  else {
+    id = Next_floatparm++;
+    Floatparms[id].name = name;
+    Floatparms[id].val = default_value;
+    Floatparms[id].default_val = default_value;
+    Floatparms[id].min = min_value;
+    Floatparms[id].max = max_value;
+  }
+  return id;
+}  /* init_floatparm */
+
 /*************
  *
  *    init_stringparm()
@@ -353,6 +399,14 @@ void fprint_options(FILE *fp)
   }
   fprintf(fp, "\n\n");
 
+  j = 0;
+  for (i = 0; i < Next_floatparm; i++) {  /* print parms */
+    fprintf(fp, "assign(");
+    fprintf(fp, "%s, \"%.3f\"). ", Floatparms[i].name, Floatparms[i].val);
+    fprintf(fp, "%s", (++j % 3 == 0) ? "\n" : "");
+  }
+  fprintf(fp, "\n\n");
+
   j = 0;
   for (i = 0; i < Next_stringparm; i++) {  /* print stringparms */
     struct stringparm *sp = &(Stringparms[i]);
@@ -456,6 +510,23 @@ BOOL over_parm_limit(int value, int parm_id)
     return value > limit;
 }  /* over_parm_limit */
 
+/*************
+ *
+ *   floatparm()
+ *
+ *************/
+
+/* DOCUMENTATION
+This integer routine returns the value of a parameter.  If the floatparm
+index is out of range, bad things can happen.
+*/
+
+/* PUBLIC */
+double floatparm(int floatparm_id)
+{
+  return Floatparms[floatparm_id].val;
+}  /* floatparm */
+
 /*************
  *
  *   stringparm()
@@ -540,6 +611,16 @@ void update_flag(FILE *fout, int id, BOOL val, BOOL echo)
 	    }
 	    assign_parm(p->id, p->val, echo);
 	  }
+	  else if (p->type == FLOATPARMT) {
+	    if (echo) {
+	      fprintf(fout, "    %% %s(%s) -> assign(%s, \"%.3f\").\n",
+		      val ? "set" : "clear",
+		      Flags[id].name,
+		      Floatparms[p->id].name,
+		      p->dval);
+	    }
+	    assign_floatparm(p->id, p->dval, echo);
+	  }
 	  else {
 	    /* assume it's a stringparm */
 	    if (echo) {
@@ -655,6 +736,41 @@ void assign_parm(int id, int val, BOOL echo)
   }
 }  /* assign_parm */
 
+/*************
+ *
+ *   assign_floatparm()
+ *
+ *************/
+
+/* DOCUMENTATION
+This routine assigns a value to a floatparm.  The floatparm is identified by
+its integer ID (which is available from str_to_floatparm_id()).
+If the ID is not valid, or if the value is out of range, a
+fatal error occurs.
+*/
+
+/* PUBLIC */
+void assign_floatparm(int id, double val, BOOL echo)
+{
+  if (id < 0 || id >= Next_floatparm) {
+    fprintf(stdout, "assign_floatparm: id %d, is out of range.\n", id);
+    fprintf(stderr, "assign_floatparm: id %d, is out of range.\n", id);
+    fatal_error("assign_floatparm");
+  }
+  else if (val < Floatparms[id].min || val > Floatparms[id].max) {
+    fprintf(stdout, "assign_floatparm: parm %s, value %.3f out of range [%.3f..%.3f].\n",
+	    Floatparms[id].name, val, Floatparms[id].min, Floatparms[id].max);
+    fprintf(stderr, "assign_floatparm: parm %s, value %.3f out of range [%.3f..%.3f].\n",
+	    Floatparms[id].name, val, Floatparms[id].min, Floatparms[id].max);
+    fatal_error("assign_floatparm");
+  }
+  else {
+    Floatparms[id].val = val;
+    Option_updates++;
+    /* Currently, nothing depends on floatparms. */
+  }
+}  /* assign_floatparm */
+
 /*************
  *
  *   assign_stringparm()
@@ -738,6 +854,27 @@ int str_to_parm_id(char *name)
   return -1;
 }  /* str_to_parm_id */
 
+/*************
+ *
+ *   str_to_floatparm_id()
+ *
+ *************/
+
+/* DOCUMENTATION
+This routine converts the string name of a parm to its integer ID.
+If the string name is not valid, -1 is returned.
+*/
+
+/* PUBLIC */
+int str_to_floatparm_id(char *name)
+{
+  int i;
+  for (i = 0; i < Next_floatparm; i++)
+    if (str_ident(name, Floatparms[i].name))
+      return i;
+  return -1;
+}  /* str_to_floatparm_id */
+
 /*************
  *
  *   str_to_stringparm_id()
@@ -795,6 +932,24 @@ char *parm_id_to_str(int id)
   return Parms[id].name;
 }  /* parm_id_to_str */
 
+/*************
+ *
+ *   floatparm_id_to_str()
+ *
+ *************/
+
+/* DOCUMENTATION
+Given a floatparm ID, return the corresponding name of the floatparm.
+*/
+
+/* PUBLIC */
+char *floatparm_id_to_str(int id)
+{
+  if (id < 0 || id >= Next_floatparm)
+    fatal_error("floatparm_id_to_str, bad id");
+  return Floatparms[id].name;
+}  /* floatparm_id_to_str */
+
 /*************
  *
  *   stringparm_id_to_str()
@@ -899,6 +1054,29 @@ void flag_parm_dependency(int id, BOOL val, int dep_id, int dep_val)
   Flags[id].dependencies = append_dep(Flags[id].dependencies, dep);
 }  /* flag_parm_dependency */
 
+/*************
+ *
+ *   flag_floatparm_dependency()
+ *
+ *************/
+
+/* DOCUMENTATION
+This routine declares that a floatparm depends on a flag.
+If flag "id" gets value "val", then parm "dep_id" is automatically
+given value "dep_val".
+*/
+
+/* PUBLIC */
+void flag_floatparm_dependency(int id, BOOL val, int dep_id, double dep_val)
+{
+  Optdep dep = get_optdep();
+  dep->type = FLOATPARMT;
+  dep->id = dep_id;
+  dep->dval = dep_val;
+  dep->flag_trigger = val;
+  Flags[id].dependencies = append_dep(Flags[id].dependencies, dep);
+}  /* flag_floatparm_dependency */
+
 /*************
  *
  *   flag_parm_dep_default()
@@ -922,6 +1100,29 @@ void flag_parm_dep_default(int id, BOOL val, int dep_id)
   Flags[id].dependencies = append_dep(Flags[id].dependencies, dep);
 }  /* flag_parm_dep_default */
 
+/*************
+ *
+ *   flag_floatparm_dep_default()
+ *
+ *************/
+
+/* DOCUMENTATION
+This routine declares that a parm depends on a flag.
+If flag "id" gets value "val", then parm "dep_id" is automatically
+given its default value.
+*/
+
+/* PUBLIC */
+void flag_floatparm_dep_default(int id, BOOL val, int dep_id)
+{
+  Optdep dep = get_optdep();
+  dep->type = FLOATPARMT;
+  dep->id = dep_id;
+  dep->dval = Floatparms[dep_id].default_val;
+  dep->flag_trigger = val;
+  Flags[id].dependencies = append_dep(Flags[id].dependencies, dep);
+}  /* flag_floatparm_dep_default */
+
 /*************
  *
  *   parm_flag_dependency()
@@ -1058,6 +1259,21 @@ int parm_default(int parm_id)
   return Parms[parm_id].default_val;
 }  /* parm_default */
 
+/*************
+ *
+ *   floatparm_default()
+ *
+ *************/
+
+/* DOCUMENTATION
+*/
+
+/* PUBLIC */
+int floatparm_default(int floatparm_id)
+{
+  return Floatparms[floatparm_id].default_val;
+}  /* floatparm_default */
+
 /*************
  *
  *   stringparm1_default()
