@@ -27,6 +27,7 @@ static char Quote_char = '\"';
 static int Quantifier_precedence = 0;  /* see declare_quantifier_precedence */
 
 static BOOL Parenthesize = FALSE;
+static BOOL Check_for_illegal_symbols = TRUE;
 
 /* Token types */
 
@@ -1481,8 +1482,8 @@ void arrange_term(String_buf sb, Term t, int par_prec)
     else if (ARITY(t) == 1 &&
 	     unary_parse_type(sn_to_str(SYMNUM(t)), &op_prec, &type)) {
 
-      int p1 = (type == PREFIX_PAREN ||
-		type == POSTFIX_PAREN ? op_prec-1 : op_prec);
+      int p1 = ((type == PREFIX_PAREN || type == POSTFIX_PAREN) ?
+		op_prec-1 : op_prec);
 
       if (op_prec > par_prec || Parenthesize)
 	sb_append(sb, "(");
@@ -1665,12 +1666,48 @@ void remember_multiple_char_special_syms(char *str)
   if (special_char(*str) &&
       strlen(str) > 1 &&
       !string_member_plist(str, Multiple_char_special_syms)) {
-    /* printf("Multiple: %s\n", str); */
     Multiple_char_special_syms =
       plist_prepend(Multiple_char_special_syms, str);
   }
 }  /* remember_multiple_char_special_syms */
 
+/*************
+ *
+ *   forget_multiple_char_special_syms()
+ *
+ *************/
+
+static
+void forget_multiple_char_special_syms(char *str)
+{
+  /* If string is multiple-char-special, add it to a global list
+     (which is used for parsing).
+  */
+  if (special_char(*str) &&
+      strlen(str) > 1 &&
+      string_member_plist(str, Multiple_char_special_syms)) {
+    Multiple_char_special_syms =
+      plist_remove_string(Multiple_char_special_syms, str);
+  }
+}  /* forget_multiple_char_special_syms */
+
+/*************
+ *
+ *   look_for_illegal_symbols()
+ *
+ *************/
+
+static
+void look_for_illegal_symbols(char *str)
+{
+  if (strlen(str) > 1 && string_of_repeated('-', str)) {
+    printf("bad string: %s\n", str);
+    fatal_error("operations that are strings of repeated \"-\" are not allowed");
+  }
+  else if (strlen(str) > 1 && string_of_repeated('\'', str))
+    fatal_error("operations that are strings of repeated \"\'\" are not allowed");
+}  /* look_for_illegal_symbols */
+
 /*************
  *
  *   declare_parse_type()
@@ -1684,8 +1721,14 @@ void remember_multiple_char_special_syms(char *str)
 void declare_parse_type(char *str, int precedence, Parsetype type)
 {
   char *str2 = process_quoted_symbol(str);  /* if quoted, remove quotes */
+
+  if (Check_for_illegal_symbols)
+    look_for_illegal_symbols(str2);
   set_parse_type(str2, precedence, type);
-  remember_multiple_char_special_syms(str2);
+  if (type == NOTHING_SPECIAL)
+    forget_multiple_char_special_syms(str2);
+  else
+    remember_multiple_char_special_syms(str2);
 }  /* declare_parse_type */
 
 /*************
@@ -1723,17 +1766,24 @@ void declare_standard_parse_types(void)
   declare_parse_type(imp_sym(),  800, INFIX);
   declare_parse_type(impby_sym(),800, INFIX);
   declare_parse_type(or_sym(),   790, INFIX_RIGHT);
+  declare_parse_type("||",       790, INFIX_RIGHT);  /* special-purpose */
   declare_parse_type(and_sym(),  780, INFIX_RIGHT);
+  declare_parse_type("&&",       780, INFIX_RIGHT);  /* special-purpose */
 			    
-  declare_quantifier_precedence(750);  /* special case */
+  declare_quantifier_precedence(750);  /* special case for quantifiers */
 
   declare_parse_type(eq_sym(),   700, INFIX);
   declare_parse_type(neq_sym(),  700, INFIX);
   declare_parse_type("==",       700, INFIX);
+  declare_parse_type("!==",      700, INFIX);
   declare_parse_type("<",        700, INFIX);
   declare_parse_type("<=",       700, INFIX);
   declare_parse_type(">",        700, INFIX);
   declare_parse_type(">=",       700, INFIX);
+  declare_parse_type("@<",       700, INFIX);
+  declare_parse_type("@<=",      700, INFIX);
+  declare_parse_type("@>",       700, INFIX);
+  declare_parse_type("@>=",      700, INFIX);
 			    
   declare_parse_type("+",        500, INFIX);
   declare_parse_type("*",        500, INFIX);
@@ -1742,8 +1792,6 @@ void declare_standard_parse_types(void)
   declare_parse_type("\\",       500, INFIX);
   declare_parse_type("^",        500, INFIX);
   declare_parse_type("v",        500, INFIX);
-  declare_parse_type("++",       500, INFIX);
-  declare_parse_type("--",       500, INFIX);
 
   declare_parse_type(not_sym(),    350, PREFIX);
   declare_parse_type("\'",       300, POSTFIX);
@@ -1916,3 +1964,18 @@ void parenthesize(BOOL setting)
   Parenthesize = setting;
 }  /* parenthesize */
 
+/*************
+ *
+ *   check_for_illegal_symbols()
+ *
+ *************/
+
+/* DOCUMENTATION
+*/
+
+/* PUBLIC */
+void check_for_illegal_symbols(BOOL setting)
+{
+  Check_for_illegal_symbols = setting;
+}  /* check_for_illegal_symbols */
+
