@@ -45,29 +45,18 @@
   do that now (with shared or with nonshared variables), just free
   the old one and get a new one.
 
-  With this code, you can switch back and forth with conditional
-  compilation: #define SHARE_VARIABLES.
-
   Things to be careful about:
 
   (1) Don't change variable numbers of variable terms.  Just use
   get_variable_term and free_term.
   (2) Don't use the container field of variables.
 
-  If you observe those rules, I think everything will be okay; in
-  particular, you should get the same results with and without
-  SHARE_VARIABLES, except for memory usage.
-
   January 29, 2003.
 */
 
 /* Private definitions and types */
 
-#define SHARE_VARIABLES
-
-#ifdef SHARE_VARIABLES
 static Term Shared_variables[MAX_VNUM];
-#endif
 
 /*
  * memory management
@@ -124,10 +113,8 @@ free all of the subterms as well, call zap_term(t) instead.
 /* PUBLIC */
 void free_term(Term p)
 {
-#ifdef SHARE_VARIABLES
   if (VARIABLE(p))
     return;  /* variables are never freed, because they are shared */
-#endif
   Arg_mem -= p->arity;
   free_mem(p, PTRS_TERM + p->arity);
   Term_frees++;
@@ -199,29 +186,22 @@ be an integer >= 0.
 /* PUBLIC */
 Term get_variable_term(int var_num)
 {
-#ifdef SHARE_VARIABLES
-  if (var_num < 0 || var_num >= MAX_VNUM || var_num >= MAX_VAR)
-    fatal_error("get_variable_term: var_num too big");
+  if (var_num < 0 || var_num >= MAX_VAR)
+    fatal_error("get_variable_term: var_num out of range");
 
-  if (Shared_variables[var_num] == NULL) {
-    /* Get a term that will stay throughout the process. */
+  if (var_num >= MAX_VNUM || Shared_variables[var_num] == NULL) {
     Term t = malloc(sizeof(struct term));
     t->private_symbol = var_num;
     t->arity = 0;
     t->private_flags = 0;
     t->container = NULL;
     t->u.id = 0;
-    Shared_variables[var_num] = t;
+    if (var_num >= MAX_VNUM)
+      return t;  /* will not be shared */
+    else
+      Shared_variables[var_num] = t;
   }
   return Shared_variables[var_num];
-#else
-  Term t = get_term(0);
-
-  if (var_num < 0 || var_num >= MAX_VAR)
-    fatal_error("get_variable_term,  var_num out of range.");
-  t->private_symbol = var_num;
-  return t;
-#endif
 }  /* get_variable_term */
 
 /*************
@@ -259,13 +239,15 @@ instead.
 /* PUBLIC */
 Term get_rigid_term(char *sym, int arity)
 {
+  Term t1;
+  int sn;
   if (arity > MAX_ARITY) {
     printf("\nArity %d requested for symbol \"%s\"\n", arity, sym);
     fprintf(stderr, "\nArity %d requested for symbol \"%s\".\n", arity, sym);
     fatal_error("get_rigid_term, arity too big");
   }
-  Term t1 = get_term(arity);
-  int sn = str_to_sn(sym, arity);
+  t1 = get_term(arity);
+  sn = str_to_sn(sym, arity);
   if (sn >= MAX_SYM)
     fatal_error("get_rigid_term, too many symbols");
   t1->private_symbol = -sn;
@@ -995,10 +977,10 @@ integer as the constant symbol.
 /* PUBLIC */
 Term nat_to_term(int n)
 {
-  char s[100];
+  char s[25];
   if (n < 0)
     fatal_error("nat_to_term: negative term");
-  return get_rigid_term(itoa(n, s), 0);
+  return get_rigid_term(int_to_str(n, s, 25), 0);
 }  /* nat_to_term */
 
 /*************
@@ -1015,14 +997,50 @@ representation.
 /* PUBLIC */
 Term int_to_term(int i)
 {
-  char s[100];
+  char s[25];
   Term t;
-  t = get_rigid_term(itoa(abs(i), s), 0);
+  t = get_rigid_term(int_to_str(abs(i), s, 25), 0);
   if (i < 0)
     t = build_unary_term_safe("-", t);
   return t;
 }  /* int_to_term */
 
+/*************
+ *
+ *   bool_to_term()
+ *
+ *************/
+
+/* DOCUMENTATION
+This routine takes an Bool and returns the Term
+representation.
+*/
+
+/* PUBLIC */
+Term bool_to_term(BOOL val)
+{
+  return get_rigid_term( val ? true_sym() : false_sym() , 0);
+}  /* bool_to_term */
+
+/*************
+ *
+ *   double_to_term()
+ *
+ *************/
+
+/* DOCUMENTATION
+This routine takes a double and returns
+a constant Term with the string representation of the
+double as the constant symbol.
+*/
+
+/* PUBLIC */
+Term double_to_term(double d)
+{
+  char s[25];
+  return get_rigid_term(double_to_str(d, s, 25), 0);
+}  /* double_to_term */
+
 /*************
  *
  *   natural_constant_term(t)
@@ -1152,6 +1170,110 @@ BOOL term_to_int(Term t, int *result)
     return FALSE;
 }  /* term_to_int */
 
+/*************
+ *
+ *   term_to_double()
+ *
+ *************/
+
+/* DOCUMENTATION
+Given a term, see if it represents a double.
+If so, set *result to the double and return TRUE.
+If not, return FALSE.
+*/
+
+/* PUBLIC */
+BOOL term_to_double(Term t, double *result)
+{
+  if (CONSTANT(t))
+    return str_to_double(sn_to_str(SYMNUM(t)), result); 
+  else
+    return FALSE;
+}  /* term_to_double */
+
+/*************
+ *
+ *   term_to_number()
+ *
+ *************/
+
+/* DOCUMENTATION
+Given a term, see if it represents an integer or a double.
+If so, set *result (a double) to the number and return TRUE.
+If not, return FALSE.
+*/
+
+/* PUBLIC */
+BOOL term_to_number(Term t, double *result)
+{
+  int i;
+  if (term_to_int(t, &i)) {
+    *result = (double) i;
+    return TRUE;
+  }
+  else if (term_to_double(t, result))
+    return TRUE;
+  else
+    return FALSE;
+}  /* term_to_number */
+
+/*************
+ *
+ *   true_term()
+ *
+ *************/
+
+/* DOCUMENTATION
+*/
+
+/* PUBLIC */
+BOOL true_term(Term t)
+{
+  return is_term(t, true_sym(), 0);
+}  /* true_term */
+
+/*************
+ *
+ *   false_term()
+ *
+ *************/
+
+/* DOCUMENTATION
+*/
+
+/* PUBLIC */
+BOOL false_term(Term t)
+{
+  return is_term(t, false_sym(), 0);
+}  /* false_term */
+
+/*************
+ *
+ *   term_to_bool()
+ *
+ *************/
+
+/* DOCUMENTATION
+Given a term, see if it represents an Boolean value.
+If so, set *result to the value and return TRUE.
+If not, return FALSE.
+*/
+
+/* PUBLIC */
+BOOL term_to_bool(Term t, BOOL *result)
+{
+  if (true_term(t)) {
+    *result = TRUE;
+    return TRUE;
+  }
+  else if (false_term(t)) {
+    *result = FALSE;
+    return TRUE;
+  }
+  else
+    return FALSE;
+}  /* term_to_bool */
+
 /*************
  *
  *   symbols_in_term()
