@@ -59,7 +59,7 @@ int next_available_multiplier()
       Multipliers[i] = TRUE;
       return i;
     }
-  fatal_error("next_available_multiplier, none avaiable (infinite loop?).");
+  fatal_error("next_available_multiplier, none available (infinite loop?).");
   return -1;  /* to quiet compiler */
 }  /* next_available_multiplier */
 
