@@ -43,11 +43,13 @@ Symbol_data *Sn_to_mace_sn;
 int Number_of_cells;
 struct cell *Cells;           /* the table of cells (dynamically allocated) */
 struct cell **Ordered_cells;  /* (pointers to) permutation of Cells */
+int First_skolem_cell;
 
 /* Misc Variables*/
 
 int Domain_size;
 Term *Domain;    /* array of terms representing (shared) domain elements  */
+BOOL Skolems_last;
 
 Plist Ground_clauses;  /* Mclauses (see ground.h) */
 
@@ -292,6 +294,8 @@ void initialize_for_search(Plist clauses)
   if (flag(Opt->arithmetic))
     init_arithmetic();
 
+  Skolems_last = flag(Opt->skolems_last);
+
   /* Collect data for each symbol. */
 
   Max_domain_element_in_input = -1;
@@ -377,7 +381,7 @@ void init_for_domain_size(void)
       c->possible[j] = Domain[j];  /* really just a flag */
   }
 
-  order_cells(flag(Opt->skolems_last), flag(Opt->verbose));
+  order_cells(flag(Opt->verbose));
   
   if (flag(Opt->negprop))
     init_negprop_index();
