@@ -74,8 +74,8 @@ static struct {
 
   // other stuff
 
-  Plist desc_to_be_disabled;  // Descendents of these to be disabled
-  Plist cac_clauses;          // Clauses that trigger back CAC check
+  Plist desc_to_be_disabled;   // Descendents of these to be disabled
+  Plist cac_clauses;           // Clauses that trigger back CAC check
 
   BOOL searching;      // set to TRUE when first given is selected
   BOOL initialized;    // has this structure been initialized?
@@ -116,6 +116,7 @@ Prover_options init_prover_options(void)
   p->pos_ur_resolution      = init_flag("pos_ur_resolution",      FALSE);
   p->neg_ur_resolution      = init_flag("neg_ur_resolution",      FALSE);
   p->paramodulation         = init_flag("paramodulation",         FALSE);
+  p->eval_rewrite           = init_flag("eval_rewrite",           FALSE);
 
   p->ordered_res            = init_flag("ordered_res",            TRUE);
   p->check_res_instances    = init_flag("check_res_instances",    FALSE);
@@ -177,6 +178,7 @@ Prover_options init_prover_options(void)
   p->auto_process           = init_flag("auto_process",            TRUE);
   p->auto2                  = init_flag("auto2",                  FALSE);
   p->raw                    = init_flag("raw",                    FALSE);
+  p->production             = init_flag("production",             FALSE);
 
   p->lex_order_vars         = init_flag("lex_order_vars",         FALSE);
 
@@ -207,6 +209,7 @@ Prover_options init_prover_options(void)
   p->random_part       = init_parm("random_part",          0,      0,INT_MAX);
   p->random_seed       = init_parm("random_seed",          0,     -1,INT_MAX);
   p->eval_limit        = init_parm("eval_limit",        1024,     -1,INT_MAX);
+  p->eval_var_limit    = init_parm("eval_var_limit",      -1,     -1,INT_MAX);
 
   p->max_depth =        init_parm("max_depth",            -1,     -1,INT_MAX);
   p->lex_dep_demod_lim =init_parm("lex_dep_demod_lim",    11,     -1,INT_MAX);
@@ -226,6 +229,7 @@ Prover_options init_prover_options(void)
   p->depth_penalty =    init_floatparm("depth_penalty",         0.0,-DBL_LARGE,DBL_LARGE);
   p->var_penalty =      init_floatparm("var_penalty",           0.0,-DBL_LARGE,DBL_LARGE);
   p->default_weight =   init_floatparm("default_weight",  DBL_LARGE,-DBL_LARGE,DBL_LARGE);
+  p->complexity =       init_floatparm("complexity",            0.0,-DBL_LARGE,DBL_LARGE);
 
   p->sos_limit =        init_parm("sos_limit",         20000,     -1,INT_MAX);
   p->sos_keep_factor =  init_parm("sos_keep_factor",       3,      2,10);
@@ -241,7 +245,6 @@ Prover_options init_prover_options(void)
 
   p->max_weight =   init_floatparm("max_weight",  100.0, -DBL_LARGE, DBL_LARGE);
 
-
   // STRINGPARMS:
   // (internal-name, external-name, number-of-strings, str1, str2, ... )
   // str1 is always the default
@@ -440,6 +443,13 @@ Prover_options init_prover_options(void)
   flag_parm_dep_default(p->raw, FALSE, p->backsub_check);
   flag_flag_dep_default(p->raw, FALSE, p->lightest_first);
   flag_stringparm_dep_default(p->raw, FALSE, p->literal_selection);
+
+  // production mode
+
+  flag_flag_dependency(p->production,   TRUE,  p->raw,               TRUE);
+  flag_flag_dependency(p->production,   TRUE,  p->eval_rewrite,      TRUE);
+  flag_flag_dependency(p->production,   TRUE,  p->hyper_resolution,  TRUE);
+  flag_flag_dependency(p->production,   TRUE,  p->back_subsume,     FALSE);
   
   return p;
   
@@ -1126,19 +1136,29 @@ void clause_wt_with_adjustments(Topform c)
 static
 void cl_process_simplify(Topform c)
 {
-  if (!clist_empty(Glob.demods)) {
+  if (flag(Opt->eval_rewrite)) {
+    int count = 0;
+    clock_start(Clocks.demod);
+    rewrite_with_eval(c);
+    if (flag(Opt->print_gen)) {
+      printf("rewrites %d:     ", count);
+      fwrite_clause(stdout, c, CL_FORM_STD);
+    }
+    clock_stop(Clocks.demod);
+  }
+  else if (!clist_empty(Glob.demods)) {
     if (flag(Opt->lex_order_vars)) {
       renumber_variables(c, MAX_VARS);
       c->normal_vars = FALSE;  // demodulation can make vars non-normal
     }
     clock_start(Clocks.demod);
-    demodulate_clause(c,
-		      parm(Opt->demod_step_limit),
-		      parm(Opt->demod_increase_limit),
-		      !flag(Opt->quiet),
-		      flag(Opt->lex_order_vars));
+      demodulate_clause(c,
+			parm(Opt->demod_step_limit),
+			parm(Opt->demod_increase_limit),
+			!flag(Opt->quiet),
+			flag(Opt->lex_order_vars));
     if (flag(Opt->print_gen)) {
-      printf("demod:     ");
+      printf("rewrite:     ");
       fwrite_clause(stdout, c, CL_FORM_STD);
     }
     clock_stop(Clocks.demod);
@@ -2111,7 +2131,8 @@ void init_search(void)
 	      floatparm(Opt->prop_atom_weight),
 	      floatparm(Opt->nest_penalty),
 	      floatparm(Opt->depth_penalty),
-	      floatparm(Opt->var_penalty));
+	      floatparm(Opt->var_penalty),
+	      floatparm(Opt->complexity));
 
   if (Glob.given_selection == NULL)
     Glob.given_selection = selector_rules_from_options(Opt);
@@ -2219,7 +2240,8 @@ void init_search(void)
   resolution_options(flag(Opt->ordered_res),
 		     flag(Opt->check_res_instances),
 		     flag(Opt->initial_nuclei),
-		     parm(Opt->ur_nucleus_limit));
+		     parm(Opt->ur_nucleus_limit),
+		     flag(Opt->eval_rewrite));
 		     
   paramodulation_options(flag(Opt->ordered_para),
 			 flag(Opt->check_para_instances),
@@ -2268,7 +2290,9 @@ void index_and_process_initial_clauses(void)
 	     flag(Opt->back_demod_hints),
 	     demodulate_clause);
   init_semantics(Glob.interps, Clocks.semantics,
-		 stringparm1(Opt->multiple_interps), parm(Opt->eval_limit));
+		 stringparm1(Opt->multiple_interps),
+		 parm(Opt->eval_limit),
+		 parm(Opt->eval_var_limit));
 
   // Do Sos and Denials last, in case we PROCESS_INITIAL_SOS.
 
@@ -2292,7 +2316,7 @@ void index_and_process_initial_clauses(void)
   ////////////////////////////////////////////////////////////////////////////
   // Demodulators
 
-  if (!clist_empty(Glob.demods)) {
+  if (!clist_empty(Glob.demods) && !flag(Opt->eval_rewrite)) {
     fflush(stdout);
     bell(stderr);
     fprintf(stderr,
@@ -2305,53 +2329,70 @@ void index_and_process_initial_clauses(void)
   for (p = Glob.demods->first; p != NULL; p = p->next) {
     Topform c = p->c;
     assign_clause_id(c);
-    if (!pos_eq_unit(c->literals))
-      fatal_error("input demodulator is not equation");
-    else {
-      int type;
-      if (flag(Opt->dont_flip_input))
-	orient_equalities(c, FALSE);  /* don't allow flips */
-      else
-	c = orient_input_eq(c);  /* this replaces c if any flipping occurs */
-      if (c->justification->next != NULL) {
-	printf("\nNOTE: input demodulator %d has been flipped.\n", c->id);
-	fflush(stdout);
-	fprintf(stderr, "\nNOTE: input demodulator %d has been flipped.\n",
-		c->id);
-	if (flag(Opt->bell))
-	  bell(stderr);
-	fflush(stderr);
-      }
-      type = demodulator_type(c,
-			      parm(Opt->lex_dep_demod_lim),
-			      flag(Opt->lex_dep_demod_sane));
-      if (flag(Opt->dont_flip_input) &&
-	  type != ORIENTED &&
-	  !renamable_flip_eq(c->literals->atom)) {
-	type = ORIENTED;  /* let the user beware */
+    if (flag(Opt->eval_rewrite)) {
+      if (c->is_formula) {
+	/* make it into a pseudo-clause */
+	c->literals = new_literal(TRUE, formula_to_term(c->formula));
+	upward_clause_links(c);
+	zap_formula(c->formula);
+	c->formula = NULL;
+	c->is_formula = FALSE;
+	clause_set_variables(c, MAX_VARS);
 	mark_oriented_eq(c->literals->atom);
-	bell(stderr);
-	fprintf(stderr,"\nWARNING: demodulator does not satisfy term order\n");
-	fflush(stderr);
-	printf("\nWARNING: demodulator does not satisfy term order: ");
-	f_clause(c);
-	fflush(stdout);
       }
-      else if (type == NOT_DEMODULATOR) {
-	Term a = ARG(c->literals->atom,0);
-	Term b = ARG(c->literals->atom,1);
-	printf("bad input demodulator: "); f_clause(c);
-	if (term_ident(a, b))
-	  fatal_error("input demodulator is instance of x=x");
-	else if (!variables_subset(a, b) && !variables_subset(b, a))
-	  fatal_error("input demoulator does not have var-subset property");
+    }
+    else {
+      if (!pos_eq_unit(c->literals))
+	fatal_error("input demodulator is not equation");
+      else {
+	int type;
+	if (flag(Opt->dont_flip_input))
+	  orient_equalities(c, FALSE);  /* don't allow flips */
 	else
-	  fatal_error("input demoulator not allowed");
+	  c = orient_input_eq(c);  /* this replaces c if any flipping occurs */
+	if (c->justification->next != NULL) {
+	  printf("\nNOTE: input demodulator %d has been flipped.\n", c->id);
+	  fflush(stdout);
+	  fprintf(stderr, "\nNOTE: input demodulator %d has been flipped.\n",
+		  c->id);
+	  if (flag(Opt->bell))
+	    bell(stderr);
+	  fflush(stderr);
+	}
+	type = demodulator_type(c,
+				parm(Opt->lex_dep_demod_lim),
+				flag(Opt->lex_dep_demod_sane));
+	if (flag(Opt->dont_flip_input) &&
+	    type != ORIENTED &&
+	    !renamable_flip_eq(c->literals->atom)) {
+	  type = ORIENTED;  /* let the user beware */
+	  mark_oriented_eq(c->literals->atom);
+	  bell(stderr);
+	  fprintf(stderr,"\nWARNING: demodulator does not satisfy term order\n");
+	  fflush(stderr);
+	  printf("\nWARNING: demodulator does not satisfy term order: ");
+	  f_clause(c);
+	  fflush(stdout);
+	}
+	else if (type == NOT_DEMODULATOR) {
+	  Term a = ARG(c->literals->atom,0);
+	  Term b = ARG(c->literals->atom,1);
+	  printf("bad input demodulator: "); f_clause(c);
+	  if (term_ident(a, b))
+	    fatal_error("input demodulator is instance of x=x");
+	  else if (!variables_subset(a, b) && !variables_subset(b, a))
+	    fatal_error("input demoulator does not have var-subset property");
+	  else
+	    fatal_error("input demoulator not allowed");
+	}
+	index_demodulator(c, type, INSERT, Clocks.index);
       }
-      index_demodulator(c, type, INSERT, Clocks.index);
     }
   }
 
+  if (flag(Opt->eval_rewrite))
+    init_dollar_eval(Glob.demods);
+
   ////////////////////////////////////////////////////////////////////////////
   // Hints
   
@@ -2470,9 +2511,11 @@ void index_and_process_initial_clauses(void)
     fwrite_clause_clist(stdout,Glob.sos,     CL_FORM_STD);
     fwrite_demod_clist(stdout,Glob.demods,   CL_FORM_STD);
   }
-  if (Glob.hints->length > 0)
-    printf("\n%% %d hints processed (%d are redundant).\n",
-	   Glob.hints->length, redundant_hints());
+  if (Glob.hints->length > 0) {
+      int redundant = redundant_hints();
+      printf("\n%% %d hints (%d processed, %d redundant).\n",
+	     Glob.hints->length - redundant, Glob.hints->length, redundant);
+    }
 
   print_separator(stdout, "end of clauses for search", TRUE);
 
